/**
 * PGDate.java
 *
 * @author Created by Donald Fraser
 */

package org.postgresql.util;

import java.sql.Date;

/**
 * Class to catpure SQL Dates that inlcude negative infinity and infinity
 */
public class PGDate extends Date
{
	boolean m_bIsInfinite = false;
	
	public PGDate( long lDate)
	{
		super(lDate);
	}
	
	public PGDate( boolean bPostiveInfinity)
	{
		super(0);
		setInfinite(bPostiveInfinity);
	}
	
	public void setTime(long lTime)
	{
		m_bIsInfinite = false;
		super.setTime(lTime);
	}
	
	public boolean isInfinite() {return m_bIsInfinite;}

	public boolean isNegativeInfinity()
	{
		return (m_bIsInfinite == true && getTime() == Long.MIN_VALUE);
	}

	public boolean isInfinity()
	{
		return (m_bIsInfinite == true && getTime() == Long.MAX_VALUE);
	}
	
	public void setInfinite(boolean bPostiveInfinity)
	{
		if (bPostiveInfinity)
		{
			super.setTime(Long.MAX_VALUE);
		}
		else
			super.setTime(Long.MIN_VALUE);
		m_bIsInfinite = true;
	}
	

    /**
     * Tests to see if this <code>PGDate</code> object is
     * equal to the given <code>Date</code> object.
	 * Note: This implementation tries to detect possible other
	 * implementations of <code>Date</code> that know about
	 * infinity. It assumes that the implementation will use the
	 * same maximum and minimum values for the internal storage
	 * of the <code>java.util.Date</code> class. This can easily
	 * be changed if it turns out not to be the case.
     *
     * @param ts the <code>Date</code> value to compare with
     * @return <code>true</code> if the given <code>Date</code>
     *         object is equal to this <code>PGDate</code> object;
     *         <code>false</code> otherwise
     */
	public boolean equals(Date sqldate)
	{
		if (super.equals(sqldate))
		{
			if (m_bIsInfinite)
			{
				if (sqldate.toString().compareToIgnoreCase(toString()) != 0)
					return false;
			}
			return true;
		}
		else
		{
			return false;
		}
	}

    /**
     * Indicates whether this <code>PGDate</code> object is
     * earlier than the given <code>Date</code> object.
	 * Note: This implementation tries to detect possible other
	 * implementations of <code>Date</code> that know about
	 * infinity. It assumes that the implementation will use the
	 * same maximum and minimum values for the internal storage
	 * of the <code>java.util.Date</code> class. This can easily
	 * be changed if it turns out not to be the case.
     *
     * @param ts the <code>Date</code> value to compare with
     * @return <code>true</code> if this <code>PGDate</code> object is
	 * earlier; <code>false</code> otherwise
     */
	
    public boolean before(Date sqlDate)
	{
		if (super.before(sqlDate))
		{
			return true;
		}
		else
		{
			// look for the case when time values are equal
			// but we are negative infinity and the timestamp value supplied is
			// not negative infinity
			if ( super.equals(sqlDate) &&
				 !equals(sqlDate) &&
				 m_bIsInfinite &&
				 getTime() == Long.MIN_VALUE )
				return true;
			return false;
		}
    }

    /**
     * Indicates whether this <code>PGDate</code> object is
     * later than the given <code>Date</code> object.
	 * Note: This implementation tries to detect possible other
	 * implementations of <code>Date</code> that know about
	 * infinity. It assumes that the implementation will use the
	 * same maximum and minimum values for the internal storage
	 * of the <code>java.util.Date</code> class. This can easily
	 * be changed if it turns out not to be the case.
     *
     * @param ts the <code>Date</code> value to compare with
     * @return <code>true</code> if this <code>PGDate</code> object
	 * is later; <code>false</code> otherwise
     */
    public boolean after(Date sqlDate)
	{
		if (super.after(sqlDate))
		{
	    	return true;
		}
		else
		{
			// look for the case when time values are equal
			// but we are infinity and the timestamp value supplied is
			// not infinity
			if ( super.equals(sqlDate) &&
				 !equals(sqlDate) &&
				 m_bIsInfinite &&
				 getTime() == Long.MAX_VALUE)
				return true;
			return false;
	    }
    }
	
    /**
     * Compares this <code>PGDate</code> object to the given
	 * <code>Date</code> object.
	 * Note: This implementation tries to detect possible other
	 * implementations of <code>Date</code> that know about
	 * infinity. It assumes that the implementation will use the
	 * same maximum and minimum values for the internal storage
	 * of the <code>java.util.Date</code> class. This can easily
	 * be changed if it turns out not to be the case.
     *
     * @param   ts   the <code>Date</code> object to be compared to
	 *                this <code>PGDate</code> object
     * @return  the value <code>0</code> if the two <code>Date</code>
	 *          objects are equal; a value less than <code>0</code> if this
     *          <code>PGDate</code> object is before the given argument;
	 *          and a value greater than <code>0</code> if this
	 *          <code>PGDate</code> object is after the given argument.
     */
    public int compareTo(Date sqlDate)
	{
        int i;
		if (equals(sqlDate))
			i = 0;
		else if (before(sqlDate))
			i = -1;
		else
			i = 1;
        return i;
    }
	
	public String toString()
	{
		if (m_bIsInfinite)
		{
			if (getTime() == (Long.MAX_VALUE))
				return "infinity";
			return "-infinity";
		}
		else
			return super.toString();
	}
}

